# python3-opsi
# This code is part of the opsi.org project
# http://opsi.org
#
# Copyright (c) uib gmbh (www.uib.de)
# This sourcecode is owned by the uib gmbh, D55118 Mainz, Germany
#
# This source code is part of co-funding project, and so are
# ALL RIGHTS RESEVERD right now
# http://uib.de/en/opsi_cofunding
# Once the co-funding is finished it will be released as full open source

if not bootimageVersion or (int(bootimageVersion) < 20160517):
    raise Exception(u"This product requires opsi-linux-bootimage version >= 20160517")
try:
    if not backend.backend_info()['opsiVersion'].startswith('4.'):
        raise ValueError('No opsi 4.x backend present')
except Exception:
    raise Exception(u"This is a opsi 4.x product")

isOpsi43 = False
if backend.backend_info()['opsiVersion'].startswith('4.3'):
    isOpsi43 = True

if isOpsi43:
    show_message(u"Installing Product: %s" % (productId), 10)

try:
    import ConfigParser
except:
    import configparser
from OPSI.Util.File import IniFile
from OPSI.Util.File import TextFile
from OPSI.Backend.BackendManager import BackendManager
from OPSI.Backend.JSONRPC import JSONRPCBackend
from OPSI.Util import md5sum
from hashlib import md5
try:
    from twisted.conch.ssh import keys
except:
    pass

#include opsi setup library
depot.copy(SCRIPT_PATH+'/opsisetuplib.py','/tmp')
exec(open('/tmp/opsisetuplib.py').read())

initOpsiSetupLib()

checkModules('local_imaging')

logger.notice(u"##############################################################")
initOpsiSetupLib()
logger.notice(u"opsi-local-image-delimage.py")
logger.notice(u"##############################################################")
productVersion =''
packageVersion =''
for i in range(len(productOnDepots)):
    if (productOnDepots[i]['productId'] == productId):
        productVersion = productOnDepots[i]['productVersion']
        packageVersion = productOnDepots[i]['packageVersion']
    break
if isOpsi43:
    show_message("Installing Product: %s_%s-%s on %s" % (productId,productVersion,packageVersion,clientId), 10)
logger.notice(u"Installing Product: %s_%s-%s on %s" % (productId,productVersion,packageVersion,clientId))
scriptMessageSubject.setMessage(u"Installing Product: %s_%s-%s on %s" % (productId,productVersion,packageVersion,clientId))
logger.notice(u"##############################################################")

useGpt = False
inUefi = inUefiMode()
if inUefi:
    useGpt = True

systemPartitionNumber = 1
systemPartitionLabel  = 'SYSTEM'
systemPartitionSizeM = 0

dataPartitionSizeM = 0
dataPartitionNumber = 0
dataPartitionLabel  = 'sysdata'

swapPartitionSizeM = 4000
winpePartitionNumber = 2
swapPartitionSize   = "4000M"
swapPartitionLabel  = "swap"
swapPartitionLetter = "X"

backupPartitionNumber = 3
backupPartitionCreate = True
backupPartitionLabel  = "opsidata"

if inUefi:
    systemPartitionNumber += 2
    winpePartitionNumber += 2
    backupPartitionNumber += 2

logger.notice("Partition numbers are now: system=%d, data=%s, swap=%d, backup=%d" % (systemPartitionNumber, dataPartitionNumber, winpePartitionNumber, backupPartitionNumber))

# get multiDiskMode and backup_partition_on_same_disk from opsi-local-image-prepare
multiDiskMode  = '0'
backup_partition_on_same_disk = True
if hasattr(backend, "productPropertyState_getValues"):
    # opsi >= 4.3
    for property_id, values in (
        backend.productPropertyState_getValues(product_ids=['opsi-local-image-prepare'], object_ids=[clientId], with_defaults=True)[clientId]['opsi-local-image-prepare']
        .items()
    ):
        logger.info("Found product property: %s=%r", property_id, values)
        if property_id == 'multi_disk_mode':
            multiDiskMode  = values[0]
        if property_id == 'backup_partition_on_same_disk':
            backup_partition_on_same_disk = values[0]
else:
    backend.backend_setOptions( { 'addProductPropertyStateDefaults': True } )
    pps = backend.productPropertyState_getObjects(objectId = clientId, productId = 'opsi-local-image-prepare',propertyId = 'multi_disk_mode')
    if pps:
        for state in pps:
            myvalues = state.getValues()
            for key in myvalues:
                multiDiskMode  = key
    pps = backend.productPropertyState_getObjects(objectId = clientId, productId = 'opsi-local-image-prepare',propertyId = 'backup_partition_on_same_disk')
    if pps:
        for state in pps:
            myvalues = state.getValues()
            for key in myvalues:
                backup_partition_on_same_disk  = key
    backend.backend_setOptions( { 'addProductPropertyStateDefaults': False } )

logger.notice("multiDiskMode: %s ; backup_partition_on_same_disk: %s" % (multiDiskMode,backup_partition_on_same_disk))

if isOpsi43:
    show_message("Reading disks", 15)
logger.notice(u"Swapoff all to avoid disk in use .....")
execute("swapoff -a -v")
# Get harddisks
disks = getHarddisks()
disks = sorted(disks, key=lambda disk: disk.device)
# Get harddisks
disks = getHarddisks()
disks = sorted(disks, key=lambda disk: disk.device)
showDiskOrder(disks)
logger.notice(u"Normally we using disk 0")
diskindex = getDiskIndexFromMultiDiskMode(multiDiskMode)
disk = disks[diskindex]
logger.notice(u"Using disk device %s" % (disk.device))
# check if (backup_partition_on_same_disk == False) and only one disk, what would make no sense
# and if so fall back to (backup_partition_on_same_disk == true)
backupdiskindex, backup_partition_on_same_disk = getBackupdiskindexFromBackupPartitionOnSameDisk(backup_partition_on_same_disk)
logger.notice(u"backup_partition_on_same_disk: %s - backupdiskindex %s" % (backup_partition_on_same_disk,backupdiskindex))
if (backup_partition_on_same_disk == False):
    # so we have a backup disk 
    backupPartitionNumber = 1
    if useGpt:
        # we assume that the backup disk is also gpt
        backupPartitionNumber += 2

logger.notice("Partition numbers are now: system=%d, data=%s, swap=%d, backup=%d" % (systemPartitionNumber, dataPartitionNumber, winpePartitionNumber, backupPartitionNumber))

if not useGpt:
    # Display partition information
    partitions = disk.getPartitions()
    for p in partitions:
        fs   = p.get('fs', 'unknown')
        mytype = p.get('type', 'unknown')
        ui.getMessageBox().addText( u"Partition %s: Start %s MB, End %s MB, Filesystem %s, Type %s.\n" \
                                % (     p['number'],
                                        p['start']/(1000*1000),
                                        p['end']/(1000*1000),
                                        fs,
                                        mytype) )
else:
    execute('/usr/sbin/sgdisk -p %s' % (disk.device))

imageDir = '/mntimg'
prodDir = '/mntprod'
loopDir = '/mntloop'
goOn = True

checkOliPartitions(diskindex,backupdiskindex,useGpt,backup_partition_on_same_disk)
backupdisk = disks[backupdiskindex]
checkCreateDir(imageDir)
try:
    testPartitionEx(backupdisk.device, backupPartitionNumber, useGpt)
except:
    if isOpsi43:
        show_error("Could not find partition %d on disk: %s . - Please restart this product (opsi-local-image-prepare)" % (backupPartitionNumber,backupdisk.device))
    raise Exception(u"Could not find partition %d on disk: %s . - Please restart this product (opsi-local-image-prepare)" % (backupPartitionNumber,backupdisk.device))

partitionLabel = partitionHasLabel(backupdisk.device, backupPartitionNumber)
if (partitionLabel == backupPartitionLabel):
    logger.notice(u"%s%s exists as expected - fine" % (backupdisk.device, backupPartitionNumber) )
else:
    logger.critical(u"%s%s exists not as expected with label %s - not ok - giving up" % (backupdisk.device, backupPartitionNumber, backupPartitionLabel) )
    scriptMessageSubject.setMessage(u"%s%s exists not as expected with label %s - not ok - giving up" % (backupdisk.device, backupPartitionNumber, backupPartitionLabel) )
    if isOpsi43:
        show_error("Wrong partition table - Please run the opsi product 'opsi-local-image-prepare' to prepare this disk !")
    raise Exception(u"Wrong partition table - Please run the opsi product 'opsi-local-image-prepare' to prepare this disk ! ")

# check if backup partition is in use
checkClearMountpoint(imageDir, backupdisk.device, backupPartitionNumber)
checkClearPartition(imageDir, backupdisk.device, backupPartitionNumber)
checkMount(imageDir, backupdisk.device, backupPartitionNumber)


# list image dir
if not os.path.exists(imageDir):
    logger.warning(u"Could not find image dir: %s - bad.." % (imageDir) )
    if isOpsi43:
        show_error("Error: Could not find image dir: %s - Aborting" % (imageDir))
    raise Exception(u"Error: Could not find image dir: %s - Aborting" % (imageDir) )
else:
    logger.notice(u"List usage in image directory: %s" % (imageDir))
    execute('du -hs %s/*' % imageDir)


imageFileOrg = productProperties.get('imagefile', '')


# check if we want to delete all
if (imageFileOrg == 'all_images'):
    logger.notice(u"Will delete all existing backup images")
    
    all_entries = os.listdir(imageDir)
    logger.notice(u"List all entries: %s" % (all_entries))
    dirListToDelete = [entry for entry in all_entries if os.path.isdir('%s/%s' % (imageDir, entry))]
    logger.notice(u"List all entries: %s" % (dirListToDelete))
    if 'lost+found' in dirListToDelete:
        dirListToDelete.remove('lost+found')

    ## del image base directory
    #logger.notice(u"Deleting: %s " % (imageDir) )
    #execute('rm -r %s/*' % imageDir)
#
    ## correct imagefiles_list: we have now a empty list
    #myvalues = []
    #backend.productPropertyState_create(productId =  'opsi-local-image-restore',propertyId = 'imagefiles_list',objectId = clientId,values = myvalues)


else:
    # do not (!) delete all images
    if (imageFileOrg == ''):
        logger.warning(u"Error: No Image file name given - Aborting")
        #raise Exception(u"Error: No Image file name given - Aborting")
    dirListToDelete = []
    dirListToDelete.append(imageFileOrg)

for imageFileDel in dirListToDelete:    
    imageFile =forceUnicode(re.sub("[\s\<\>\?\"\:\|\\\/\*]", "_",imageFileDel))
    logger.notice(u"Original image file name >%s< changed to internal image file name >%s<" % (imageFileDel, imageFile))

    if isOpsi43:
        show_message("Will delete image: %s " % imageFileDel)
    scriptMessageSubject.setMessage( u"Will delete image: %s " % imageFileDel)
    logger.notice(u"Will delete image: %s " % imageFile)
    goOn = True


    if goOn:
        # list image dir
        if not os.path.exists(imageDir):
            logger.warning(u"Could not find image dir: %s - bad.." % (imageDir) )
            if isOpsi43:
                show_error("Error: Could not find image dir: %s - Aborting" % (imageDir) )
            raise Exception(u"Error: Could not find image dir: %s - Aborting" % (imageDir) )
        else:
            logger.notice(u"List usage in image directory: %s" % (imageDir))
            execute('du -hs %s/*' % imageDir)

    if (imageFileOrg == ''):
        logger.critical(u"Error: No Image file name given - Aborting")
        if isOpsi43:
            show_error("Error: No Image file name given - Aborting")
        raise Exception(u"Error: No Image file name given - Aborting")


    if goOn:
        # del image
        deldir = '%s/%s' % (imageDir, imageFile)
        if not os.path.exists(deldir):
            logger.warning(u"Could not find image dir: %s - bad.." % (deldir) )
            if isOpsi43:
                show_error("Error: Could not find image dir: %s - Aborting" % (deldir) )
            raise Exception(u"Error: Could not find image dir: %s - Aborting" % (deldir) )
        else:
            if isOpsi43:
                show_message("Deleting: %s " % (deldir), 80)
            logger.notice(u"Deleting: %s " % (deldir) )
            execute('rm -r %s' % deldir)

    # correct imagefiles_list
    pps = backend.productPropertyState_getObjects(objectId = clientId, productId = 'opsi-local-image-restore',propertyId = 'imagefiles_list')
    if pps:
        for state in pps:
            myvalues = state.getValues()
            myValuesBefore = myvalues
            myvaluesType = type(myvalues)

            if (imageFile in myvalues):
                myvalues.remove(imageFile)
                myvalues = forceList(myvalues)
                logger.notice(u"Removed imagefile: %s from property imagefiles_list" % (imageFile) )
                backend.productPropertyState_create(productId =  'opsi-local-image-restore',propertyId = 'imagefiles_list',objectId = clientId,values = myvalues)
            else:
                logger.notice(u"imagefile: %s is not found in imagefiles_list" % (imageFile) )

if goOn:
    # update available space on backup partition
    logger.notice(u"disk space free on %s" % (imageDir))
    dfoutput = execute("df -h %s" % (imageDir))
    backend.productPropertyState_create(productId = 'opsi-local-image-backup', propertyId = "free_on_backup", objectId = clientId, values = dfoutput)

# Set product installation status
productOnClient.setInstallationStatus('installed')
productOnClient.setActionProgress('')
productOnClient.setActionResult('successful')
productOnClient.setLastAction('setup')

# Reboot
reboot()
